//-----------------------------------------------------------------------------
//
// Classes for implementing a graphical equalizer
//
//-----------------------------------------------------------------------------

/*
	VT Framework License (BSL 1.0)

	Copyright (C) 2007 Sergey A. Galin 
	E-mail: sergey.galin@gmail.com

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/

#include "vt_eqgraphic.hpp"
#include "vt_debug.hpp"

using namespace VT;

/*
	float srate2 = srate*0.5f;
	for(int i=0; i<out.getSize(); i++)
		out.set(i, T(dB( float(i)*srate2 )));
*/


void CFrequencyResponse::dBtoVolume(const CVTDPVector& in, CVTDPVector &out)
{
	out.resize(in.getSize());
	for(size_t i=0; i<in.getSize(); i++)
		out.set(i, Convert_dBTo01_NoClip(in.get(i)));
}


void CSinFR::fill_dB(CVTDPVector &out)
{
/*
	enum TFRBezierType{ kLowShelf, kHighShelf, kBand };
	double fFreq, fShift, fWidth;
	TFRBezierType type;
*/
	//for(size_t i=0; i<out.getSize(); i++)
	//	//out.set( i, 2.0 * cos(4.0*kPi*double(i)/double(out.getSize()))*double(fGain) );
	//	out.set( i, Convert_01TodB_NoClip(fGain) );	

	

	/*SDWORD sep = URound( double(out.getSize()-1) * fFreq / fMaxFrequency );
	for(i=0; i<sep; i++) out.set( i, 0);
	for(;i<SDWORD(out.getSize()); i++) out.set( i, 10 );
	return;*/

	/*OPENDBGLOG;
	DBGLOGP("Freq = %f Hz --- ", fFreq);
	DBGLOGP("Index = %d\n", int(hz2index(fFreq, out)));
	CLOSEDBGLOG;*/

	double range, slopestartfreq, slopeendfreq, hz;


	switch(type){
	case kLowShelf:	
		{
		
			// Convert slope range in octaves into frequency range
			range = pow(2.0, fWidth/2.0); // 1/2 of band to left, 1/2 of band to right
			slopestartfreq = fFreq / range;
			slopeendfreq = fFreq * range;
			for(size_t i = 0; i<out.getSize(); i++){
				hz = index2hz(i, out);
				if(hz<slopestartfreq)
					out.set(i, fGain_dB);
				else if(hz>slopeendfreq)
					out.set(i, 0);
				else
					out.set(i, fGain_dB * (1.0-sin( kPi * Log2(hz / fFreq) / fWidth ))/2.0 );
			}


			/*{SDWORD shelfpoint = hz2index(fFreq, out);
			for(i=0; i<shelfpoint && i<SDWORD(out.getSize()); i++)
				out.set(i, fGain_dB);
			for(; i<SDWORD(out.getSize()); i++)
				out.set(i, 0);
			return; }*/

			return;
		}
	case kHighShelf: 
		{
			range = pow(2.0, fWidth/2.0); // 1/2 of band to left, 1/2 of band to right
			slopestartfreq = fFreq / range;
			slopeendfreq = fFreq * range;
			for(size_t i = 0; i<out.getSize(); i++){
				hz = index2hz(i, out);
				if(hz<slopestartfreq)
					out.set(i, 0);
				else if(hz>slopeendfreq)
					out.set(i, fGain_dB);
				else
					out.set(i, fGain_dB * (1.0+sin( kPi * Log2(hz / fFreq) / fWidth ))/2.0 );
			}


			/*SDWORD i;
			SDWORD shelfpoint = hz2index(fFreq, out);
			for(i=0; i<shelfpoint && i<SDWORD(out.getSize()); i++)
				out.set(i, 0);
			for(; i<SDWORD(out.getSize()); i++)
				out.set(i, fGain_dB);*/
			return; 
		}
	default: /*case kBand:*/ 
		{
			// Align fFreq to the closest available FR sample
			double frsamplestep = FRSampleStepHz(out);
			double shift = fmod(fFreq, frsamplestep);
			double freak;
			size_t peakindex;
			if( shift<frsamplestep/2.0 ){
				freak = fFreq-shift;
				peakindex = size_t(URound(floor( fFreq/frsamplestep )));
			}else{
				freak = fFreq-shift+1.0;
				peakindex = size_t(URound(ceil( fFreq/frsamplestep )));
			}



			range = pow(2.0, fWidth); // band to left, band to right
			slopestartfreq = freak / range;
			slopeendfreq = freak * range;
			for(size_t i = 0; i<out.getSize(); i++){

				if(i == peakindex)
					out.set(i, fGain_dB);
				else{

					hz = index2hz(i, out);
					
					if(hz<slopestartfreq || hz>slopeendfreq)
						out.set(i, 0);
					else
						out.set(i, fGain_dB * (1.0+cos( kPi * Log2(hz / freak) / fWidth ))/2.0 );
				}
			}

			/*SDWORD i;
			SDWORD shelfpoint = hz2index(fFreq, out);
			for(i=0; i<shelfpoint-25 && i<SDWORD(out.getSize()); i++)
				out.set(i, 0);
			for(; i<shelfpoint+25 && i<SDWORD(out.getSize()); i++)
				out.set(i, fGain_dB);
			for(; i<SDWORD(out.getSize()); i++)
				out.set(i, 0);*/
			return;	
		}
	};	
}