//-----------------------------------------------------------------------------
//
// Classes for implementing a graphical equalizer
//
//-----------------------------------------------------------------------------

/*
	VT Framework License (BSL 1.0)

	Copyright (C) 2007 Sergey A. Galin 
	E-mail: sergey.galin@gmail.com

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/

#ifndef VT_EQGRAPHIC_HPP
#define VT_EQGRAPHIC_HPP

#include "vt_audiomath.hpp"
#include "vt_buffers.hpp"

namespace VT {

	class CFrequencyResponse
	{
	public:
		double
			fMaxFrequency;

		// Write frequency graphic into the table.
		virtual void fill_dB(CVTDPVector &out)=0;

		static void dBtoVolume(const CVTDPVector& in, CVTDPVector &out);

		// Writes frequency graphic into 'out', then converts
		// all values from dB to amplitude.
		void fill(CVTDPVector &out){ fill_dB(out); dBtoVolume(out, out); }

	protected:

		VTINLINE SDWORD hz2index( double hz, const CVTDPVector& fr ){ 
			return URound( double(fr.getSize()-1) * hz / fMaxFrequency ); }

		VTINLINE double index2hz( size_t index, const CVTDPVector& fr ){ 
			return double(index) * fMaxFrequency / double(fr.getSize()-1); }
			//return double(index) * fMaxFrequency * (1.0-1.0/double(fr.getSize()-1)) / double(fr.getSize()-1); }
		
		VTINLINE double FRSampleStepHz( const CVTDPVector& fr ){ 
			return fMaxFrequency / double(fr.getSize()-1);
		}
	};


	// Do not modify, it will break GUI's
	enum TSinFRType{ kLowShelf=0, kBand=1, kHighShelf=2  };

	class CSinFR: public CFrequencyResponse
	{
	public:	
		double 
			fFreq, 
			fGain_dB, 
			fWidth; 

		TSinFRType type;
		bool bEnabled;

		CSinFR(): fFreq(0), fGain_dB(0), fWidth(0), bEnabled(true){}

		virtual void fill_dB(CVTDPVector &out);

	protected:
	};

} // namespace

#endif


