//-----------------------------------------------------------------------------
//
// A number of classes which expand functionality of VST GUI and
// provide a cleaner programming interface.
//
//-----------------------------------------------------------------------------

/*
	VT Framework License (BSL 1.0)

	Copyright (C) 2007 Sergey A. Galin 
	E-mail: sergey.galin@gmail.com

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/

#include <stdio.h>
#include <ctype.h>
#include "vt_vstguiaddons.hpp"
#include "vt_assert.hpp"
#include "vt_messageboxes.hpp"
#include "vt_fileutils.hpp" // used by CVTBitmap
#include "vt_string.hpp"

#if defined(VT_WINDOWS)
	#include "windows.h"
	#include <shlobj.h>
	#include <shellapi.h>	
	#include <commdlg.h>
#endif

#if USE_LIBPNG
	#include "png.h"
#endif

using namespace VT;

static const CColor 
	IndColor={ 15, 225, 206, 0 },
	AlarmColor={ 225, 174, 112, 0 };

void VTStdTextConvert(char *input, char *string, void *userDta)
{
	//Params::CParameter* param = ((Params::CParameter*)(userDta));

	strcpy(string, input);

}

void VTStdString2FloatConvert(char *string, float &output)
{
	double x;
	int parsed = sscanf(string, "%lf", &x);
	if( parsed ) 
		output = float(x);
}

void VTStdStringConvert(float value, char *string, void *userDta)
{
	((Params::CParameter*)(userDta))->textFromVisible( value, string );
}





void CVTVSTGUIControl::setParam(Params::CParameter *newparam) 
{ 
	vtparam = newparam;
	VTSetProperTag(); 
	VTUpdateFromEffect(); 
}

void CVTVSTGUIControl::setRangeFromParamValue()
{
	control->setMin( vtparam->getMin() );
	control->setMax( vtparam->getMax() );
	control->setDefaultValue( vtparam->getDefault() );
}

void CVTVSTGUIControl::setRangeFromParamVisibleValue()
{
	control->setMin( vtparam->getMinVisibleValue() );
	control->setMax( vtparam->getMaxVisibleValue() );
	control->setDefaultValue( vtparam->getDefault_VisibleValue() );
}

void CVTVSTGUIControl::VTUpdateEffect()
{	
	vtparam->setValue( VTNewEffectValue() ); 
}









CVTStdSlider::CVTStdSlider(int x1, int y1, CControlListener *listener, Params::CParameter *prm,
	CBitmap *back, CBitmap *handle, long style):

	CVTVSTGUIControl( 0, prm ),
	CSlider( CRect(x1, y1, x1 + back->getWidth (), y1 + back->getHeight ()), 
		listener, VTCONSTRUCTIONTAG(prm), CPoint(0, 0), 
		back->getWidth(), handle, back, CPoint(0, 0), style)
{	
	VTCONTROLSCONSTRUCTION;
	setMin( 0 );
	setMax( 1 );
	setDefaultValue( getParam()->as01(getParam()->getDefault()) );
	setValue( getParam()->as01() );	
}


// Uses fragment of window background to create slider's background
CVTStdSlider::CVTStdSlider(int x1, int y1, int x2, int y2,
		CControlListener *listener, Params::CParameter *prm,
		CBitmap *windowBack, CBitmap *handle, long style):

	CVTVSTGUIControl( 0, prm ),
	CSlider(CRect(x1, y1, x2, y2), 
		listener, VTCONSTRUCTIONTAG(prm), CPoint(0, 0), 
		(style&kVertical)?y2-y1-1:x2-x1-1, 
		handle, windowBack, CPoint(x1, y1), style)
			
{	
	VTCONTROLSCONSTRUCTION;
	setMin( 0 );
	setMax( 1 );
	setDefaultValue( getParam()->as01(getParam()->getDefault()) );
	setValue( getParam()->as01() );	
}

CVTStdSlider::CVTStdSlider(const CRect& pos, CControlListener *listener, Params::CParameter *prm,
	CBitmap *windowBack, CBitmap *handle, long style):

	CVTVSTGUIControl( 0, prm ),
	CSlider( pos, 
		listener, VTCONSTRUCTIONTAG(prm), CPoint(0, 0), 
		(style&kVertical)?pos.y2-pos.y-1:pos.x2-pos.x-1, 
		handle, windowBack, CPoint(pos.x, pos.y), style)
{
	VTCONTROLSCONSTRUCTION;
	setMin( 0 );
	setMax( 1 );
	VTUpdateFromEffect();
}

bool CVTStdSlider::VTUpdateFromEffect()
{ 
	setDefaultValue(getParam()->getDefault_as01()); 
	#if defined(VT_VST_REDUCEUPDATES)
		float as01 = getParam()->as01();	
		if( getValue()!=as01 ){
			setValue( as01 );
			return true;
		}else
			return false;
	#else
		setValue( getParam()->as01() );
		return true;
	#endif
}


CVTKnob::CVTKnob(const CRect &size, CControlListener *listener, Params::CParameter *prm, CBitmap *background, CBitmap *handle):
	CVTVSTGUIControl( 0, prm ),
	CKnob(size, listener, VTCONSTRUCTIONTAG(prm), background, handle, CPoint(0, 0))
{
	VTCONTROLSCONSTRUCTION;
	setMin( 0 );
	setMax( 1 );
	setDefaultValue( getParam()->as01(getParam()->getDefault()) );
	setValue( getParam()->as01() );		
}


bool CVTKnob::VTUpdateFromEffect()
{ 
	setDefaultValue(getParam()->getDefault_as01()); 
	#if defined(VT_VST_REDUCEUPDATES)
		float as01 = getParam()->as01();
		if( getValue()!=as01 ){
			setValue( as01 );
			return true;
		}else
			return false;
	#else
		setValue(getParam()->as01());
		return true;
	#endif
}


CVTOnOffButton::CVTOnOffButton(const CRect &size, CControlListener *listener, Params::CParameter *prm,
		CBitmap *background):
	COnOffButton(size, listener, VTCONSTRUCTIONTAG(prm), background, kPostListenerUpdate),
	CVTVSTGUIControl( 0, prm )
{
	VTCONTROLSCONSTRUCTION;
	setMin( 0 );
	setMax( 1 );
	VTUpdateFromEffect();
}

bool CVTOnOffButton::VTUpdateFromEffect()
{ 
	setDefaultValue(getParam()->getDefault_as01()); 
	#if defined(VT_VST_REDUCEUPDATES)
		float as01 = getParam()->as01();
		if( getValue()!=as01 ){
			setValue( as01 );
			return true;
		}else
			return false;
	#else
		setValue( getParam()->as01() );
		return true;
	#endif
}








CVTStdTextLabel::CVTStdTextLabel (const CRect &size, CBitmap *background, const char *txt):
	CTextLabel(size, txt, background, 0)
{
	setFontColor( IndColor );
	setFont( kNormalFontVerySmall );
	setHoriAlign( kRightText );
}


CVTStdParamDisplayLabel::CVTStdParamDisplayLabel (const CRect &size,  Params::CParameter *prm, CBitmap *background):
	CVTStdTextLabel(size, background),
	CVTVSTGUIControl( 0, prm )
{
	setControl((CControl*) this);
	VTUpdateFromEffect();
}


bool CVTStdParamDisplayLabel::VTUpdateFromEffect()
{ 
	char buf[kVstMaxParamStrLen+1];	
	getParam()->getAsText(buf);	
	if( strcmp(buf, getText()) ){
		setText( buf ); 
		return true;
	}else
		return false;
}














CVTPeakDisplay::CVTPeakDisplay(const CRect &size, CBitmap *bgNormal, CBitmap *bgAlarm, float *value, float thresh):
	CTextLabel(size, "", 0, 0),
	normalBg(bgNormal), 
	alarmBg(bgAlarm),
	fAlarmThreshold(thresh),
	pValue(value),
	resetValue(0.0),
	alarmState(kVTPeakNormal)
{
	setFont( kNormalFontVerySmall );
	setHoriAlign( kRightText );
	
	bgNormal->remember();
	bgAlarm->remember();
	
	normalColor = IndColor;
	alarmColor = AlarmColor;

	updateValue(true);
}

CVTPeakDisplay::~CVTPeakDisplay()
{ 
	pBackground = 0; 
	normalBg->forget(); 
	alarmBg->forget(); 
}

void CVTPeakDisplay::setText (const char *txt)
{ 
	if( strcmp(getText(), txt) ){
		CTextLabel::setText(txt); 
		setDirty(); 
	}
}

void CVTPeakDisplay::setState(float val, bool forceUpdate)
{ 
	if(val>fAlarmThreshold) 
		setAlarmState(forceUpdate); 
	else 
		setNormalState(forceUpdate); 
}

void CVTPeakDisplay::setValue(float value, bool forceUpdate)
{ 
	char buf[kVstMaxParamStrLen+1]; 
	dB2string(value, buf);
	if(forceUpdate || strcmp(buf, getText())){
		setText(buf); 
		setState(value, forceUpdate); 
	}
}

void CVTPeakDisplay::setAlarmState(bool forceUpdate){
	if( forceUpdate || alarmState != kVTPeakAlarm ){
		setFontColor( alarmColor );
		pBackground = alarmBg;
		alarmState = kVTPeakAlarm;
		setDirty();
	}
}

void CVTPeakDisplay::setNormalState(bool forceUpdate){
	if( forceUpdate || alarmState != kVTPeakNormal ){
		setFontColor( normalColor );
		pBackground = normalBg;
		alarmState = kVTPeakNormal;
		setDirty();
	}
}












CVTStdTextEdit::CVTStdTextEdit(const CRect &sz, CControlListener *listener, Params::CParameter *prm, CBitmap *background):
	CTextEdit(sz, listener, VTCONSTRUCTIONTAG(prm), "", background),
	CVTVSTGUIControl( 0, prm )	
{
	VTCONTROLSCONSTRUCTION;
	CColor fontColor={ 0xFF, 0xFF, 0xFF, 0 }; //{ 0, 0, 0, 0 };
	setFontColor( fontColor );
	setFont( kNormalFontVerySmall );
	setHoriAlign( kRightText );
	
	//setTextEditConvert( VTStdTextConvert, getParam() );

	if( getParam()->is_dB() )
		setString2FloatConvert( VisibleTextToVisible_dB );
	else
		setString2FloatConvert( VTStdString2FloatConvert );

	setStringConvert ( VTStdStringConvert, getParam() );
	VTUpdateFromEffect();
}

float CVTStdTextEdit::VTNewInControlEffectValue()
{ 
	float tmp;
	char buf[ 1024 ]; // Stupid VST GUI has size of buffer hardcoded to 256
		//[ kVstMaxParamStrLen + 4];
	getText(buf);
	if( sscanf(buf, "%f", &tmp) ) 
		return tmp;
	else
		return getValue(); 
}

bool CVTStdTextEdit::VTUpdateFromEffect()
{ 
	// setMin( getParam()->getMin() );
	// setMax( getParam()->getMax() );
	// setDefaultValue( getParam()->getDefault() );
	setRangeFromParamVisibleValue();
		
	#if !defined(VT_VST_REDUCEUPDATES)
		setValue( getParam()->getVisibleValue() ); 	
		return true;
	#else
		float vis = getParam()->getVisibleValue();
		if( getValue()!=vis ){
			setValue( vis );
			return true;
		}else
			return false;
	#endif
}






CVTStdCombobox::CVTStdCombobox(const CRect &sz, CControlListener *listener, Params::CParameter *prm, 
							   CBitmap *background,	CBitmap *bgWhenClick, const long style):
	COptionMenu(sz, listener, VTCONSTRUCTIONTAG(prm), background, bgWhenClick, style),
	CVTVSTGUIControl( 0, prm )
{
	if( !prm->CanIterateValues() )
		throw _T("Attempt to create a drop-down control from a parameter which cannot iterate values.");

	VTCONTROLSCONSTRUCTION;
	
	setFont( kNormalFontVerySmall );
	setHoriAlign( kLeftText );
	
	UpdateOptionList();

	VTUpdateFromEffect();
}


void CVTStdCombobox::UpdateOptionList()
{
	// Clear old menu
	removeAllEntry();
	
	// Construct the menu	
	size_t n = getParam()->GetNValues();
	for(size_t i=0; i<n; i++){
		char buf[255]; // See vstcontrols.cpp - it's the buffer size there.
		// prm->getAsText(float(i), buf, sizeof(buf)-1);
		getParam()->getAsText(getParam()->GetValueByIdx(i), buf, sizeof(buf)-1);
		addEntry(buf);
	}

	// We did it!
	RangeCorrected();
}

bool CVTStdCombobox::VTUpdateFromEffect()
{
	if( ParamRangeChanged() )
		UpdateOptionList();	

	// Update value ranges
	setMin( 0.0f );
	setMax( float( getParam()->GetNValues() ) );
	setDefaultValue( float(getParam()->GetIdxOfValue( getParam()->getDefault() )) );

	float val = float(getParam()->GetIdxOfValue( getParam()->getValue() ));

	if( getValue()!=val || getCurrent()!=URound(val) ){
		setValue( val );
		setCurrent( URound(val), false );
		return true;
	}else
		return false;
}


void CVTStdCombobox::setValue( float val )
{ 
	if( val!=getValue() ) 
		COptionMenu::setValue(val); 
}
	
float CVTStdCombobox::VTNewInControlEffectValue()
{
	VTASSERT( this->getValue()>=0.0f );
	return getParam()->GetValueByIdx( size_t(URound(this->getValue())) ); 
}
	
float CVTStdCombobox::VTNewEffectValue()
{
	VTASSERT( this->getValue()>=0.0f );
	// return getParam()->fromVisibleValue( VTNewInControlEffectValue() ); 
	return getParam()->GetValueByIdx( size_t(URound(this->getValue())) );
}













int CVTStdContainer::tagToControlIdx(long tag)
{
	// Scan array of controls to find one with the tag
	for(int i=0; i<iNumControls; i++)
		if( VTControl(i)->getControl()->getTag() == tag )
			return i;
	// Holy ****
	return -1;
}
	
bool CVTStdContainer::VTUpdateAllFromEffect(){
	#if defined(VT_VST_REDUCEUPDATES)
		bool bUpdated = false;
		for(int i=0; i<iNumControls; i++)
			if( VTControl(i)->VTUpdateFromEffect() )
				bUpdated = true;
		return bUpdated;
	#else
		for(int i=0; i<iNumControls; i++)
			VTControl(i)->VTUpdateFromEffect();
		return true;
	#endif
}

void CVTStdContainer::valueChanged (CDrawContext *pContext, CControl *pControl)
{
	// Here we need to convert pControl pointer to pointer to some our very
	// special VT control. We use tags for that nasty procedure. (All tags 
	// in the controls must be unique.)

	long tag = pControl -> getTag ();

	if( tag==cVTSplashScreenTag ){
		// This is an important workaround: without this call the splash
		// screen will not be repainted by backgrpound.
		((CVTStdAEffGUIEditor*)(VTEffect()->getEditor()))->doIdleStuff();
		return; // No need to update anything else.
	}

	if( tag>= 0 && tag<cVTNotEditorTagShift ){

		int idx = tagToControlIdx( tag );
		if( idx >= 0 && idx<VTNumControls() ){
			CVTVSTGUIControl *control = VTControl( idx );
			control->VTUpdateEffect(); // Oh yeah! (C)
		}

	}
	
	VTUpdateAllFromEffect();
}


CSplashScreen* CVTStdContainer::CreateSplashScreen(const CPoint& clickPos, CBitmap *pButtonImg, 
	const CPoint& splashPos, CBitmap *pSplashImg)
{	
	if( iNumSplashScreens >= iMaxSplashScreens ){
		VTMB::Error(_T("Maximum number of splash screens hit!"), true );
		return NULL;
	}

	CRect 
		// ssz - Splash SiZe
		ssz(0, 0, pSplashImg->getWidth(), pSplashImg->getHeight()), 
		// cp - Click Position (splash activation "button")
		cp(0, 0, pButtonImg->getWidth(), pButtonImg->getHeight());
	
	// Moving splash to position
	ssz.offset( splashPos.x, splashPos.y );
	
	// Moving "button" to position
	cp.offset( clickPos.x, clickPos.y );
	
	// Adding button to GUI
	addView( new CMovieBitmap(cp, this, cVTNotEditorTagShift, pButtonImg, CPoint(0,0)) );	
	
	// Adding splash to GUI
	CSplashScreen *ret = new CSplashScreen(cp, this, cVTSplashScreenTag, pSplashImg, ssz, CPoint(0,0));	
	addView( ret );

	// Registering the splash in the internal splash array
	pSplashScreens[ iNumSplashScreens++ ] = ret;	

	return ret;
}

CSplashScreen* CVTStdContainer::CreateSplashScreen(const CPoint& clickPos, long iButtonResId, 
		const CPoint& splashPos, long iSplashResId)
{
	CBitmap
		*pButtonImg = new CBitmap( iButtonResId ),
		*pSplashImg = new CBitmap( iSplashResId );
	CSplashScreen* ret = CreateSplashScreen( clickPos, pButtonImg, splashPos, pSplashImg );
	pButtonImg->forget();
	pSplashImg->forget();
	return ret;
}










CVTStdContainer::CVTStdContainer(const CRect &size, CFrame *pParent, CBitmap *pBackground, 
	CVTAudioEffectX *effct, int maxCtls, int maxSplashes):
	
	CViewContainer(size, pParent, pBackground),
	effect(effct),
	// pSplashScreen(NULL),
	iMaxControls( maxCtls ),
	iNumControls( 0 ),
	iMaxSplashScreens( maxSplashes ),
	iNumSplashScreens( 0 )
{
	pControls = (iMaxControls)? new CVTVSTGUIControl* [ iMaxControls ] : 0;
	memset( pControls, 0, sizeof(CVTVSTGUIControl*) * iMaxControls );

	pSplashScreens = (maxSplashes)? new CSplashScreen* [ iMaxSplashScreens ] : 0;
	memset( pSplashScreens, 0, sizeof(CSplashScreen*) * iMaxSplashScreens );
}


CVTStdContainer::~CVTStdContainer()
{
	delete pControls; 
	pControls = NULL;
	delete pSplashScreens;
	pSplashScreens = NULL;
}


CVTStdAEffGUIEditor::CVTStdAEffGUIEditor(CVTAudioEffectX* effct):
	AEffGUIEditor((AudioEffect*)effct),
	backgroundBitmap(0),
	objectContainer(0),
	vtInIdle( false )
{	
}

CVTStdAEffGUIEditor::CVTStdAEffGUIEditor(CVTAudioEffectX* effct, int bgbitmapid):
	AEffGUIEditor((AudioEffect*)effct),
	backgroundBitmap(0),
	objectContainer(0),
	vtInIdle( false )
{ 
	VTSetBackground( bgbitmapid );	
}

void CVTStdAEffGUIEditor::VTSetBackground(CBitmap *bmp)
{
	backgroundBitmap = bmp;
	rect.left   = 0;
	rect.top    = 0;
	rect.right  = (VstInt16)backgroundBitmap->getWidth ();
	rect.bottom = (VstInt16)backgroundBitmap->getHeight ();
}

void CVTStdAEffGUIEditor::VTSetBackground(int bgbitmapid)
{ 
	if( backgroundBitmap ) 
		backgroundBitmap->forget();
	VTSetBackground( new CBitmap(bgbitmapid) );
}


CVTStdAEffGUIEditor::~CVTStdAEffGUIEditor()
{	
	if( backgroundBitmap ) 
		backgroundBitmap->forget(); 
}



bool CVTStdAEffGUIEditor::open (void *ptr)
{
	AEffGUIEditor::open (ptr);
	CRect size (rect.left , rect.top, rect.right+1, rect.bottom+1);	
	CFrame* frm = new CFrame (size, ptr, this);
	frm->removeAll ();
	frm->setBackground (backgroundBitmap);	
	objectContainer = CreateContainer(size, frm);
	frm->addView( objectContainer );
	frm->setDirty ();
	// Setting newly created frame
	frame = frm;

	VTEffect()->editorOpen();

	return true;
}

void CVTStdAEffGUIEditor::close()
{ 
	try{
		if( frame ){ 
			delete frame ; 
			frame  = 0; 
		}
	}catch(...){
		frame = 0; 
		VTDEBUGMSGT("CVTStdAEffGUIEditor::close(): failed to delete frame.");
	}

	objectContainer = 0; 

	VTEffect()->editorClosed();
}	

void CVTStdAEffGUIEditor::idle(){ 	
	if(!vtInIdle){
		vtInIdle = true;
		if(objectContainer) 
			objectContainer->onIdle(); 			
		AEffGUIEditor::idle(); 	
		vtInIdle = false;
	}
}












CVTRMSVUMeter::CVTRMSVUMeter(const CRect &size, CBitmap *onBitmap, CBitmap *rmsBitmp, CBitmap *offBitmap,
				long nbLed, const long style): 
	CVuMeter(size, onBitmap, offBitmap, nbLed, style),
	rmsBitmap(rmsBitmp), 
	rmsValue(0), 
	rmsOldValue(0)
{ 
	if(rmsBitmap) 
		rmsBitmap->remember(); 
	rectRMS (size.left, size.top, size.right, size.bottom);
	setDecreaseStepValue(0.03333f);
}


void CVTRMSVUMeter::setValue(float val)
{ 
	rmsValue = 0; 
	//rmsOldValue=0; 
	CVuMeter::setValue(val); 
}

void CVTRMSVUMeter::setValue(float ampval, float rmsval)
{ 
	rmsValue = Limit01(rmsval); 
	//rmsOldValue = rmsValue; 
	CVuMeter::setValue(ampval); 
}

bool CVTRMSVUMeter::attached (CView *parent)
{
	CVuMeter::attached(parent);
	if (bUseOffscreen)
		rectRMS (0, 0, size.width (), size.height ());
	else
		rectRMS (size.left, size.top, size.right, size.bottom);
	return CControl::attached (parent);
}


void CVTRMSVUMeter::draw (CDrawContext *_pContext)
{

	CVuMeter::draw(_pContext);

	if (!onBitmap) 
		return;

	CPoint pointOn(0, 0);
	CPoint pointOff(0, 0);
	CPoint pointRMS(0, 0);
	CDrawContext *pContext = _pContext;

	bounceValue ();

	float newValue = oldValue - decreaseValue;
	if (newValue < value)
		newValue = value;
	
	newValue = Clip(newValue, 0.0f, 0.998f);
	
	oldValue = newValue;

	
	rmsValue = Clip(rmsValue, 0.0f, 0.998f);
	#pragma message("vt_vstguiaddons.cpp: CVTRMSVUMeter::draw() uses hard-coded value for falloff ")	
	rmsOldValue = float(Clip( rmsOldValue-0.1, double(rmsValue), 0.998 )); //!!!!!
	//VTTestValue = rmsValue;




	bool bPeakIndVisible = true;
	
	if (bUseOffscreen)
	{
		if (!pOScreen)
		{
			pOScreen = new COffscreenContext (getFrame (), (long)size.width (), (long)size.height (), kBlackCColor);
			rectOn  (0, 0, size.width (), size.height ());
			rectOff (0, 0, size.width (), size.height ());
			rectRMS (0, 0, size.width (), size.height ());
		}
		pContext = pOScreen;
	}

	if (style & kHorizontal) 
	{
		CCoord tmp = Clip(
			(long)(((long)(nbLed * newValue + 0.5f) / (float)nbLed) * onBitmap->getWidth ()),
			long(0), long(size.width()));
		CCoord tmp2 = Clip(
			(long)(((long)(nbLed * rmsOldValue + 0.5f ) / (float)nbLed) * onBitmap->getWidth ()),
			long(0), long(size.width()));
		
		pointOff(Max(long(tmp), long(tmp2)), 0);		
		pointOn(tmp2, 0);

		if (!bUseOffscreen){
			tmp += size.left;
			tmp2 += size.left;
		}
		
		rectRMS.right = tmp2;
		rectOff.left = Max(long(tmp), long(tmp2));
		rectOn.right = tmp;
		rectOn.left = tmp2;		

		bPeakIndVisible = tmp > tmp2;
	}
	else 
	{
		CCoord tmp = Clip(
			(long)(((long)(nbLed * (getMax () - newValue) + 0.5f) / (float)nbLed) * onBitmap->getHeight ()),
			long(0), long(size.height()));
		CCoord tmp2 = Clip(
			(long)(((long)(nbLed * (getMax () - rmsOldValue) + 0.5f) / (float)nbLed) * onBitmap->getHeight ()),
			long(0), long(size.height()));
		
		pointOn (0, Min(long(tmp), long(tmp2)));
		pointRMS (0, tmp2);

		if (!bUseOffscreen){
			tmp += size.top;
			tmp2 += size.top;
		}

		rectRMS.top = tmp2;
		rectOff.bottom = Min(long(tmp), long(tmp2));
		rectOn.top     = tmp;
		rectOn.bottom  = tmp2;

		bPeakIndVisible = tmp2 > tmp;
	}

	if (offBitmap && rectOff.right>=rectOff.left && rectOff.bottom>=rectOff.top)
	{
		if( offBitmap->getWidth() >= rectOff.width()-pointOff.x &&
			offBitmap->getHeight() >= rectOff.height()-pointOff.y ){

			if (bTransparencyEnabled)
				offBitmap->drawTransparent (pContext, rectOff, pointOff);
			else
				offBitmap->draw (pContext, rectOff, pointOff);
		}
	}

	if( onBitmap && bPeakIndVisible && rectOn.right>=rectOn.left
		 && rectOn.bottom>=rectOn.top) 
	{
		if( onBitmap->getWidth() >= rectOn.width()-pointOn.x &&
			onBitmap->getHeight() >= rectOn.height()-pointOn.y ){
			if (bTransparencyEnabled)
				onBitmap->drawTransparent (pContext, rectOn, pointOn);
			else
				onBitmap->draw (pContext, rectOn, pointOn);
		}
	}

	if( rmsBitmap && rectRMS.right>=rectRMS.left && rectRMS.bottom>=rectRMS.top)
	{
		if( rmsBitmap->getWidth() >= rectRMS.width()-pointRMS.x &&
			rmsBitmap->getHeight() >= rectRMS.height()-pointRMS.y ){
			if (bTransparencyEnabled)
				rmsBitmap->drawTransparent (pContext, rectRMS, pointRMS);
			else
				rmsBitmap->draw (pContext, rectRMS, pointRMS);
		}
	}

	if (pOScreen)
		pOScreen->copyFrom (_pContext, size);
	setDirty (false);
	
}




CVTBitmap::CVTBitmap( LPCVTCHAR path ):
	CBitmap()
{
	#if WINDOWS
		VTCHAR extension[MAX_PATH+1];
		VT::ExtractFileExt( path, extension );
		if( !VTSTRICMP( extension, _T("PNG") ) ){			
			#if USE_LIBPNG				
				FILE *fp = VTFOPEN( path, _T("rb") );
				if( !fp ){
					VTDEBUGMSGT("Failed to open file.");
					return;
				}
				// setup libpng
				png_structp png_ptr;
				png_infop info_ptr;
				png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
				if (png_ptr)
				{
					info_ptr = png_create_info_struct (png_ptr);
					if (info_ptr)
					{
						if (setjmp (png_jmpbuf (png_ptr)) == 0)
						{
							int bit_depth, color_type;
							png_init_io( png_ptr, fp ); 					
							png_read_info (png_ptr, info_ptr); 					
							png_get_IHDR (png_ptr, info_ptr, 
								(png_uint_32*)&width, (png_uint_32*)&height, &bit_depth, 
								&color_type, 0, 0, 0);
							long bytesPerRow = width * (32 / 8);
							while (bytesPerRow & 0x03)
								bytesPerRow++;

							// VTDEBUGMSG(CString("Info: W=")<<long(width)<<_T(" H=")<<
							// long(height)<<_T(" BitDepth=")<<
							// long(bit_depth)<<_T(" ColorType: ")<<long(color_type)<<
							// _T(" BytesPerRow=")<<long(bytesPerRow));

							// create BITMAP
							BITMAPINFO* bmInfo = new BITMAPINFO;
							BITMAPINFOHEADER* header = (BITMAPINFOHEADER*)bmInfo;
							memset (header, 0, sizeof(BITMAPINFOHEADER));
							header->biSize = sizeof(BITMAPINFOHEADER);
							header->biWidth = width;
							header->biHeight = height;
							header->biPlanes = 1;
							header->biBitCount = 32;
							header->biCompression = BI_RGB;
							header->biClrUsed = 0;
							void* bits;
							HDC dstDC = 0; //CreateCompatibleDC (0);
							pHandle = CreateDIBSection(dstDC, bmInfo, DIB_RGB_COLORS, &bits, NULL, 0);
							VTASSERT( pHandle!=NULL );
							delete bmInfo;
							if(pHandle)
							{
								if (color_type == PNG_COLOR_TYPE_PALETTE)
									png_set_palette_to_rgb (png_ptr);
								if (color_type == PNG_COLOR_TYPE_GRAY || color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
									png_set_gray_to_rgb (png_ptr);
								if (color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8)
									png_set_gray_1_2_4_to_8 (png_ptr);
								if (png_get_valid (png_ptr, info_ptr, PNG_INFO_tRNS))
									png_set_tRNS_to_alpha (png_ptr);
								else
									png_set_filler (png_ptr, 0xFF, PNG_FILLER_AFTER);
								if (bit_depth == 16)
								{
									png_set_swap (png_ptr);
									png_set_strip_16 (png_ptr);
								}
								if (color_type == PNG_COLOR_TYPE_RGB || color_type == PNG_COLOR_TYPE_RGB_ALPHA)
									png_set_bgr (png_ptr);
								png_read_update_info (png_ptr, info_ptr);

								unsigned char** rows = new unsigned char*[1];
								rows[0] = (unsigned char*)bits + (height-1) * bytesPerRow;
								for (long i = 0; i < height; i++)
								{
									png_read_rows (png_ptr, rows, NULL, 1);
									rows[0] -= bytesPerRow;
								}
								delete [] rows;
								png_read_end (png_ptr, 0);
								// premultiply alpha
								unsigned long* pixelPtr = (unsigned long*)bits;
								for (int y = 0; y <height; y++)
								{
									for (int x = 0; x < width; x++)
									{
										unsigned char* pixel = (unsigned char*)pixelPtr;
										if (pixel[3] != 0)
										{
											pixel[0] = ((pixel[0] * pixel[3]) >> 8);
											pixel[1] = ((pixel[1] * pixel[3]) >> 8);
											pixel[2] = ((pixel[2] * pixel[3]) >> 8);
										}
										else
											*pixelPtr = 0UL;
										pixelPtr++;
									}
								}
								if (dstDC)
									DeleteDC (dstDC);
							}
						}
					}
					png_destroy_read_struct (&png_ptr, &info_ptr, NULL);
				}
				noAlpha = true; // !!!!!!!!!!
				fclose( fp );
			#endif
		}else{
			pHandle = LoadImage( 0, path, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE );
			BITMAP bm;
			if (pHandle && GetObject (pHandle, sizeof (bm), &bm))
			{
				width  = bm.bmWidth; 
				height = bm.bmHeight;
				noAlpha = true;
			}
		}

	#else
		loadFromPath( path ); // not implemented for Windows
	#endif
}









