//-----------------------------------------------------------------------------
//
// Basic Audio Types and Conversion Routines
//
//-----------------------------------------------------------------------------

/*
	VT Framework License (BSL 1.0)

	Copyright (C) 2006 Sergey A. Galin 
	E-mail: sergey.galin@gmail.com

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/


#include "vt_audiotypes.hpp"

using namespace VT;

size_t VT::GetAudioTypeSize( TSampleTypeIdx idx )
{
	switch( idx ){
		case kInt16S:				return 2;
		case kInt32S: case kFP32:	return 4;
		case kInt24S: case kInt20S:	return 3;
		case kFP64:					return 8;
	}
	return 1;
}



void VT::ConvertAudio8U_32F( const void* src, void *dest, size_t nSamples )
{
	register const TSample8U* in = (const TSample8U*)src;
	register TSample32F* out = (TSample32F*)dest;
	for( register size_t i=0; i<nSamples; i++ )
		*(out++) = TSample32F(int(*(in++))-128)/TSample32F(128.0);
}

void VT::ConvertAudio8S_32F( const void* src, void *dest, size_t nSamples )
{
	register const TSample8S* in = (const TSample8S*)src;
	register TSample32F* out = (TSample32F*)dest;
	for( register size_t i=0; i<nSamples; i++ )
		*(out++) = TSample32F(*(in++))/TSample32F(128.0);
}

void VT::ConvertAudio16S_32F( const void* src, void *dest, size_t nSamples )
{
	register const TSample16S* in = (const TSample16S*)src;
	register TSample32F* out = (TSample32F*)dest;
	for( register size_t i=0; i<nSamples; i++ )
		*(out++) = TSample32F(*(in++))/(TSample32F(kInt16SMax)+TSample32F(1.0));
}

void VT::ConvertAudio20S_32F( const void* src, void *dest, size_t nSamples )
{
	ConvertAudio24S_32F( src, dest, nSamples );
}

void VT::ConvertAudio24S_32F( const void* src, void *dest, size_t nSamples )
{
	register const unsigned char* in = (const unsigned char*)src;
	register TSample32F* out = (TSample32F*)dest;
	for( register size_t i=0; i<nSamples; i++ ){		
		*(out++) = TSample32F(ConvertSample24to32( in ))/(TSample32F(kInt32SMax)+TSample32F(1.0));
		in += 3;
	}
}

void VT::ConvertAudio32S_32F( const void* src, void *dest, size_t nSamples )
{
	register const TSample32S* in = (const TSample32S*)src;
	register TSample32F* out = (TSample32F*)dest;
	for( register size_t i=0; i<nSamples; i++ )
		*(out++) = TSample32F(*(in++))/(TSample32F(kInt32SMax)+TSample32F(1.0));
}

void VT::ConvertAudio64F_32F( const void* src, void *dest, size_t nSamples )
{
	register const TSample64F* in = (const TSample64F*)src;
	register TSample32F* out = (TSample32F*)dest;
	for( register size_t i=0; i<nSamples; i++ )
		*(out++) = TSample32F(*(in++));
}










void VT::ConvertAudio8U_64F( const void* src, void *dest, size_t nSamples )
{
	register const TSample8U* in = (const TSample8U*)src;
	register TSample64F* out = (TSample64F*)dest;
	for( register size_t i=0; i<nSamples; i++ )
		*(out++) = TSample64F(int(*(in++))-128)/TSample64F(128.0);
}

void VT::ConvertAudio8S_64F( const void* src, void *dest, size_t nSamples )
{
	register const TSample8S* in = (const TSample8S*)src;
	register TSample64F* out = (TSample64F*)dest;
	for( register size_t i=0; i<nSamples; i++ )
		*(out++) = TSample64F(*(in++))/TSample64F(128.0);
}

void VT::ConvertAudio16S_64F( const void* src, void *dest, size_t nSamples )
{
	register const TSample16S* in = (const TSample16S*)src;
	register TSample64F* out = (TSample64F*)dest;
	for( register size_t i=0; i<nSamples; i++ )
		*(out++) = TSample64F(*(in++))/(TSample64F(kInt16SMax)+TSample64F(1.0));
}

void VT::ConvertAudio20S_64F( const void* src, void *dest, size_t nSamples )
{
	ConvertAudio24S_64F( src, dest, nSamples );
}

void VT::ConvertAudio24S_64F( const void* src, void *dest, size_t nSamples )
{
	register const unsigned char* in = (const unsigned char*)src;
	register TSample64F* out = (TSample64F*)dest;
	for( register size_t i=0; i<nSamples; i++ ){		
		*(out++) = TSample64F(ConvertSample24to32( in ))/(TSample64F(kInt32SMax)+TSample64F(1.0));
		in += 3;
	}
}

void VT::ConvertAudio32S_64F( const void* src, void *dest, size_t nSamples )
{
	register const TSample32S* in = (const TSample32S*)src;
	register TSample64F* out = (TSample64F*)dest;
	for( register size_t i=0; i<nSamples; i++ )
		*(out++) = TSample64F(*(in++))/(TSample64F(kInt32SMax)+TSample64F(1.0));
}

void VT::ConvertAudio32F_64F( const void* src, void *dest, size_t nSamples )
{
	register const TSample32F* in = (const TSample32F*)src;
	register TSample64F* out = (TSample64F*)dest;
	for( register size_t i=0; i<nSamples; i++ )
		*(out++) = TSample64F(*(in++));
}






void VT::ConvertAudio32F_8U( const void* src, void *dest, size_t nSamples )
{
	register const TSample32F* in = (const TSample32F*)src;
	register TSample8U* out = (TSample8U*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample32F clip = Math::Clip( *(in++), -1.0f, 1.0f );
		*(out++) = (TSample8U)( Math::Clip<int>( Math::SRound(clip*127.0 + 128.0), 0, 255 ) );
	}
}


void VT::ConvertAudio32F_8S( const void* src, void *dest, size_t nSamples )
{
	register const TSample32F* in = (const TSample32F*)src;
	register TSample8S* out = (TSample8S*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample32F clip = Math::Clip( *(in++), -1.0f, 1.0f );
		*(out++) = TSample8S( Math::Clip<int>( Math::SRound(clip*127.0), -127, 127 ) );
	}
}

void VT::ConvertAudio32F_16S( const void* src, void *dest, size_t nSamples )
{
	register const TSample32F* in = (const TSample32F*)src;
	register TSample16S* out = (TSample16S*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample32F clip = Math::Clip( *(in++), -1.0f, 1.0f );
		*(out++) = TSample16S( Math::Clip<int>( Math::SRound(clip*32767.0), -32767, 32767 ) );
	}
}

void VT::ConvertAudio32F_20S( const void* src, void *dest, size_t nSamples )
{
	register const TSample32F* in = (const TSample32F*)src;
	register unsigned char* out = (unsigned char*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample32F clip = Math::Clip( *(in++), -1.0f, 1.0f );
		ConvertSample32to20(
			Math::Clip<TSample32S>( Math::SRound(clip*2147483647.0), -2147483647, 2147483647 ),
			out );
		out += 3;		
	}
}

void VT::ConvertAudio32F_24S( const void* src, void *dest, size_t nSamples )
{
	register const TSample32F* in = (const TSample32F*)src;
	register unsigned char* out = (unsigned char*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample32F clip = Math::Clip( *(in++), -1.0f, 1.0f );
		ConvertSample32to24(
			Math::Clip<TSample32S>( Math::SRound(clip*2147483647.0), -2147483647, 2147483647 ),
			out );
		out += 3;		
	}
}

void VT::ConvertAudio32F_32S( const void* src, void *dest, size_t nSamples )
{
	register const TSample32F* in = (const TSample32F*)src;
	register TSample32S* out = (TSample32S*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample32F clip = Math::Clip( *(in++), -1.0f, 1.0f );
		*(out++) = Math::Clip<TSample32S>( Math::SRound(clip*2147483647.0), -2147483647, 2147483647 );
	}
}









void VT::ConvertAudio64F_8U( const void* src, void *dest, size_t nSamples )
{
	register const TSample64F* in = (const TSample64F*)src;
	register TSample8U* out = (TSample8U*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample64F clip = Math::Clip( *(in++), -1.0, 1.0 );
		*(out++) = (TSample8U)( Math::Clip<int>( Math::SRound(clip*127.0 + 128.0), 0, 255 ) );
	}
}


void VT::ConvertAudio64F_8S( const void* src, void *dest, size_t nSamples )
{
	register const TSample64F* in = (const TSample64F*)src;
	register TSample8S* out = (TSample8S*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample64F clip = Math::Clip( *(in++), -1.0, 1.0 );
		*(out++) = TSample8S( Math::Clip<int>( Math::SRound(clip*127.0), -127, 127 ) );
	}
}

void VT::ConvertAudio64F_16S( const void* src, void *dest, size_t nSamples )
{
	register const TSample64F* in = (const TSample64F*)src;
	register TSample16S* out = (TSample16S*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample64F clip = Math::Clip( *(in++), -1.0, 1.0 );
		*(out++) = TSample16S( Math::Clip<int>( Math::SRound(clip*32767.0), -32767, 32767 ) );
	}
}

void VT::ConvertAudio64F_20S( const void* src, void *dest, size_t nSamples )
{
	register const TSample64F* in = (const TSample64F*)src;
	register unsigned char* out = (unsigned char*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample64F clip = Math::Clip( *(in++), -1.0, 1.0 );
		ConvertSample32to20(
			Math::Clip<TSample32S>( Math::SRound(clip*2147483647.0), -2147483647, 2147483647 ),
			out );
		out += 3;		
	}
}

void VT::ConvertAudio64F_24S( const void* src, void *dest, size_t nSamples )
{
	register const TSample64F* in = (const TSample64F*)src;
	register unsigned char* out = (unsigned char*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample64F clip = Math::Clip( *(in++), -1.0, 1.0 );
		ConvertSample32to24(
			Math::Clip<TSample32S>( Math::SRound(clip*2147483647.0), -2147483647, 2147483647 ),
			out );
		out += 3;		
	}
}

void VT::ConvertAudio64F_32S( const void* src, void *dest, size_t nSamples )
{
	register const TSample64F* in = (const TSample64F*)src;
	register TSample32S* out = (TSample32S*)dest;
	for( register size_t i=0; i<nSamples; i++ ){
		register TSample64F clip = Math::Clip( *(in++), -1.0, 1.0 );
		*(out++) = Math::Clip<TSample32S>( Math::SRound(clip*2147483647.0), -2147483647, 2147483647 );
	}
}


void VT::ConvertAudioTo32F( const void* src, TSampleTypeIdx srcType, void *dest, size_t nSamples )
{
	switch( srcType ){
		case kInt16S:	ConvertAudio16S_32F( src, dest, nSamples );	return;
		case kInt24S:	ConvertAudio24S_32F( src, dest, nSamples );	return;
		case kInt32S:	ConvertAudio32S_32F( src, dest, nSamples );	return;
		case kInt8S:	ConvertAudio8S_32F( src, dest, nSamples );	return;
		case kInt8U:	ConvertAudio8U_32F( src, dest, nSamples );	return;
		case kFP32:		memcpy( dest, src, nSamples*4 );			return;
		case kFP64:		ConvertAudio64F_32F( src, dest, nSamples );	return;
	}
	throw _T("Bad source sample type in 32-bit audio converter.");
}

void VT::ConvertAudioTo64F( const void* src, TSampleTypeIdx srcType, void *dest, size_t nSamples )
{
	switch( srcType ){
		case kInt16S:	ConvertAudio16S_64F( src, dest, nSamples );	return;
		case kInt24S:	ConvertAudio24S_64F( src, dest, nSamples );	return;
		case kInt32S:	ConvertAudio32S_64F( src, dest, nSamples );	return;		
		case kInt8S:	ConvertAudio8S_64F( src, dest, nSamples );	return;
		case kInt8U:	ConvertAudio8U_64F( src, dest, nSamples );	return;
		case kFP32:		ConvertAudio32F_64F( src, dest, nSamples );	return;
		case kFP64:		memcpy( dest, src, nSamples*8 );			return;
	}
	throw _T("Bad source sample type in 64-bit audio converter.");
}


void VT::ConvertAudio8BitStub( const void* src, void *dest, size_t nSamples )
{
	memcpy( dest, src, nSamples );
}

void VT::ConvertAudio16BitStub( const void* src, void *dest, size_t nSamples )
{
	memcpy( dest, src, nSamples*2 );
}

void VT::ConvertAudio20BitStub( const void* src, void *dest, size_t nSamples )
{
	memcpy( dest, src, nSamples*3 ); // Same as for 24 bit
}

void VT::ConvertAudio24BitStub( const void* src, void *dest, size_t nSamples )
{
	memcpy( dest, src, nSamples*3 );
}

void VT::ConvertAudio32BitStub( const void* src, void *dest, size_t nSamples )
{
	memcpy( dest, src, nSamples*4 );
}

void VT::ConvertAudio64BitStub( const void* src, void *dest, size_t nSamples )
{
	memcpy( dest, src, nSamples*8 );
}

VT::TPAudioConverterFunc VT::GetAudioConverterTo32F( TSampleTypeIdx srcType )
{
	switch( srcType ){
		case kInt16S:	return ConvertAudio16S_32F;
		case kInt24S:	return ConvertAudio24S_32F;
		case kInt32S:	return ConvertAudio32S_32F;
		case kInt8S:	return ConvertAudio8S_32F;
		case kInt8U:	return ConvertAudio8U_32F;
		case kFP32:		return ConvertAudio32BitStub;
		case kFP64:		return ConvertAudio64F_32F;
		case kInt20S:	return ConvertAudio20S_32F;
	}
	throw _T("Bad source sample type in GetAudioConverterXXXX().");
}

VT::TPAudioConverterFunc VT::GetAudioConverterTo64F( TSampleTypeIdx srcType )
{
	switch( srcType ){
		case kInt16S:	return ConvertAudio16S_64F;
		case kInt24S:	return ConvertAudio24S_64F;
		case kInt32S:	return ConvertAudio32S_64F;
		case kInt8S:	return ConvertAudio8S_64F;
		case kInt8U:	return ConvertAudio8U_64F;
		case kFP32:		return ConvertAudio32F_64F;
		case kFP64:		return ConvertAudio64BitStub;
		case kInt20S:	return ConvertAudio20S_64F;
	}
	throw _T("Bad source sample type in GetAudioConverterXXXX().");
}

VT::TPAudioConverterFunc VT::GetAudioConverterFrom32F( TSampleTypeIdx destType )
{
	switch( destType ){
		case kInt16S:	return ConvertAudio32F_16S;
		case kInt24S:	return ConvertAudio32F_24S;
		case kInt32S:	return ConvertAudio32F_32S;
		case kInt8S:	return ConvertAudio32F_8S;
		case kInt8U:	return ConvertAudio32F_8U;
		case kFP32:		return ConvertAudio32BitStub; 
		case kFP64:		return ConvertAudio32F_64F;
		case kInt20S:	return ConvertAudio32F_20S;
	}
	throw _T("Bad source sample type in GetAudioConverterXXXX().");
}

VT::TPAudioConverterFunc VT::GetAudioConverterFrom64F( TSampleTypeIdx destType )
{
	switch( destType ){
		case kInt16S:	return ConvertAudio64F_16S;		
		case kInt24S:	return ConvertAudio64F_24S;
		case kInt32S:	return ConvertAudio64F_32S;
		case kInt8S:	return ConvertAudio64F_8S;
		case kInt8U:	return ConvertAudio64F_8U;
		case kFP32:		return ConvertAudio64F_32F; 
		case kFP64:		return ConvertAudio64BitStub;
		case kInt20S:	return ConvertAudio64F_20S;
	}
	throw _T("Bad source sample type in GetAudioConverterXXXX().");
}

VT::TPAudioConverterFunc VT::GetAudioConverterStub( TSampleTypeIdx thetype )
{
	switch( GetAudioTypeSize( thetype ) ){
		case 1: return ConvertAudio8BitStub;
		case 2: return ConvertAudio16BitStub;
		case 3: return ConvertAudio24BitStub;
		case 4: return ConvertAudio32BitStub;
		case 8: return ConvertAudio64BitStub;
	}
	throw _T("Bad source sample type in GetStub().");
}




CAudioBitConverter::CAudioBitConverter():
	m_DestType( kInt8S ),
	m_SrcType( kInt8S ),
	m_pfInConverter( NULL ),
	m_pfOutConverter( NULL ),
	m_Buffer32(),
	m_Buffer64(),
	m_uChannels( 1 ),
	m_DitheringMethod( kDitheringNone )
{
}

CAudioBitConverter::CAudioBitConverter( TSampleTypeIdx srcType, TSampleTypeIdx destType, size_t bufferPrealloc,
									   TDitheringMethod dithering, unsigned nDitheringChannels ):
	m_DestType( srcType ),
	m_SrcType( destType ),
	m_pfInConverter( VT::ConvertAudio8BitStub ),
	m_pfOutConverter( VT::ConvertAudio8BitStub ),
	m_Buffer32(),
	m_Buffer64(),
	m_uChannels( 1 ),
	m_DitheringMethod( kDitheringNone )
{
	Init( srcType, destType, dithering, nDitheringChannels );
	PreallocBuffers( bufferPrealloc );
}

// Initialize source and destination conversion types
void CAudioBitConverter::Init( TSampleTypeIdx srcType, TSampleTypeIdx destType, TDitheringMethod dither, 
							  unsigned nDitheringChannels )
{
	m_SrcType = srcType;
	m_DestType = destType;
	m_uChannels = nDitheringChannels;
	m_DitheringMethod = dither; // Used by UsesDirectConversion()!

	if( m_SrcType==m_DestType ){
		// Direct copying
		m_pfInConverter = GetAudioConverterStub( m_SrcType );
		m_pfOutConverter = NULL; 
	}else{
		if( UsesDirectConversion() ){
			// Direct conversion from in to out
			m_pfOutConverter = NULL; 
			if( m_SrcType==kFP32 )
				m_pfInConverter = GetAudioConverterFrom32F( m_DestType );
			else if( m_SrcType==kFP64 )
				m_pfInConverter = GetAudioConverterFrom64F( m_DestType );
			else if( m_DestType==kFP32 )
				m_pfInConverter = GetAudioConverterTo32F( m_SrcType );
			else if( m_DestType==kFP64 )
				m_pfInConverter = GetAudioConverterTo64F( m_SrcType );
			else{
				// Error in code?
				VTDEBUGMSGT("Failed to find appropriate direct converter.");
				m_pfInConverter = NULL;
			}
		}else{
			// Buffered conversion
			if( Uses64Bit() ){
				m_pfInConverter = GetAudioConverterTo64F( m_SrcType );
				m_pfOutConverter = GetAudioConverterFrom64F( m_DestType );
			}else{
				m_pfInConverter = GetAudioConverterTo32F( m_SrcType );
				m_pfOutConverter = GetAudioConverterFrom32F( m_DestType );
			}
		}
	}

	if( m_DitheringMethod!=kDitheringNone ){
		// Setting ditherer method is a simple and fast inline operation, 
		// so this can be done in the dumb way.
		for( unsigned i=0; i<m_uChannels; i++ ){
			m_Dither32F_16S[i].SetMethod( dither );
			m_Dither64F_16S[i].SetMethod( dither );
			m_Dither32F_32S[i].SetMethod( dither );
			m_Dither64F_32S[i].SetMethod( dither );
			m_Dither32F_8S[i].SetMethod( dither );
			m_Dither64F_8S[i].SetMethod( dither );
			m_Dither32F_20S[i].SetMethod( dither );
			m_Dither64F_20S[i].SetMethod( dither );
			m_Dither32F_24S[i].SetMethod( dither );
			m_Dither64F_24S[i].SetMethod( dither );
		}
		Restart();
	}
}

void CAudioBitConverter::Restart()
{
	if( m_DitheringMethod!=kDitheringNone ){
		// Restarting ditherer is a simple and fast inline operation, 
		// so this can be done in the dumb way.
		for( unsigned i=0; i<m_uChannels; i++ ){
			m_Dither32F_16S[i].Restart();
			m_Dither64F_16S[i].Restart();
			m_Dither32F_32S[i].Restart();
			m_Dither64F_32S[i].Restart();
			m_Dither32F_8S[i].Restart();
			m_Dither64F_8S[i].Restart();
			m_Dither32F_20S[i].Restart();
			m_Dither64F_20S[i].Restart();
			m_Dither32F_24S[i].Restart();
			m_Dither64F_24S[i].Restart();
		}
	}
}

// Pre-allocate buffers for nSamples block size. Do it after Init().
void CAudioBitConverter::PreallocBuffers( size_t nSamples )
{
	if( !UsesDirectConversion() ){
		bool use64 = Uses64Bit();
		m_Buffer32.resize( use64? 0: nSamples );
		m_Buffer64.resize( use64? nSamples: 0 );
	}
}

// Convert block
void CAudioBitConverter::Convert( const void *src, void *dest, size_t nSamples )
{
	VTASSERT( src!=NULL && dest!=NULL );
	VTASSERT( m_pfInConverter!=NULL );	
	if( UsesDirectConversion() )
		m_pfInConverter( src, dest, nSamples );
	else{
		if( !UsesDithering() ){
			VTASSERT( m_pfOutConverter!=NULL );
			if( Uses64Bit() ){
				m_Buffer64.resize( nSamples );
				m_pfInConverter( src, m_Buffer64.getDataPtr(), nSamples );
				m_pfOutConverter( m_Buffer64.getDataPtr(), dest, nSamples );
			}else{
				m_Buffer32.resize( nSamples );
				m_pfInConverter( src, m_Buffer32.getDataPtr(), nSamples );
				m_pfOutConverter( m_Buffer32.getDataPtr(), dest, nSamples );
			}
		}else{
			if( nSamples % m_uChannels ){
				VTDEBUGMSGT("Number of channels doesn't divide number of samples.");
				throw _T("Number of channels doesn't divide number of samples.");
			}

			// If input is not floating point, convert to FP.
			VTASSERT( IsIntFormat( m_DestType ) && !IsFPFormat( m_DestType ) );
			const TSample64F *dithererSrc64 = NULL;
			const TSample32F *dithererSrc32 = NULL;
			bool use64 = false;
			// Configuring ditherer input data
			if( !IsFPFormat( m_SrcType ) ){
				if( Uses64Bit() ){
					m_Buffer64.resize( nSamples );
					m_pfInConverter( src, m_Buffer64.getDataPtr(), nSamples );
					use64 = true;
					dithererSrc64 = m_Buffer64.getRODataPtr();
				}else{
					m_Buffer32.resize( nSamples );
					m_pfInConverter( src, m_Buffer32.getDataPtr(), nSamples );
					use64 = false;
					dithererSrc32 = m_Buffer32.getRODataPtr();
				}
			}else{
				// Input is floating-point format - 
				// use input buffer as the source directly.
				if( m_SrcType==kFP64 ){
					use64 = true;
					dithererSrc64 = (const TSample64F*)src;
				}else{
					use64 = false;
					dithererSrc32 = (const TSample32F*)src;
				}
			}

			// Dithering
			size_t nFrames = nSamples/m_uChannels;
			#if 0 //defined(VT_CONSOLEDEBUG)
				VTPRINTF(_T("nSamples=%u Channels=%u nFrames=%u InputFormatSize=%u OutputFormatSize=%u Use64=%s\n"), 
					nSamples, m_uChannels, nFrames, GetAudioTypeSize(m_SrcType), 
					GetAudioTypeSize(m_DestType), LCYESNO(use64));

			#endif			
			switch( m_DestType ){
				case kInt16S:
					if( use64 ){
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample64F *ptr = dithererSrc64 + channel;
							TSample16S *outptr = ((TSample16S*)dest) + channel;
							for( size_t frame=0; frame<nFrames; frame++ ){
								*outptr = m_Dither64F_16S[channel].Dither( *ptr * 32767.0 );
								ptr+=m_uChannels;
								outptr+=m_uChannels;
							}
						}
					}else{
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample32F *ptr = dithererSrc32 + channel;
							TSample16S *outptr = ((TSample16S*)dest) + channel;
							for( size_t frame=0; frame<nFrames; frame++ ){
								*outptr = m_Dither32F_16S[channel].Dither( *ptr * 32767.0f );
								ptr+=m_uChannels;
								outptr+=m_uChannels;
							}
						}
					}
					break;
				case kInt24S:
					if( use64 ){
						unsigned frameSize = m_uChannels*3;
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample64F *ptr = dithererSrc64 + channel;
							unsigned char *outptr = ((unsigned char *)dest) + 3*channel;							
							for( size_t frame=0; frame<nFrames; frame++ ){
								ConvertSample32to24NoScaling( m_Dither64F_24S[channel].Dither( *ptr * 8388607.0 ), outptr );
								ptr+=m_uChannels;
								outptr+=frameSize;
							}
						}
					}else{
						unsigned frameSize = m_uChannels*3;
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample32F *ptr = dithererSrc32 + channel;
							unsigned char *outptr = ((unsigned char *)dest) + 3*channel;							
							for( size_t frame=0; frame<nFrames; frame++ ){
								ConvertSample32to24NoScaling( m_Dither32F_24S[channel].Dither( *ptr * 8388607.0f ), outptr );
								ptr+=m_uChannels;
								outptr+=frameSize;
							}
						}
					}
					break;
				case kInt32S:
					if( use64 ){
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample64F *ptr = dithererSrc64 + channel;
							TSample32S *outptr = ((TSample32S*)dest) + channel;
							for( size_t frame=0; frame<nFrames; frame++ ){
								*outptr = m_Dither64F_32S[channel].Dither( *ptr * 2147483647.0 );
								ptr+=m_uChannels;
								outptr+=m_uChannels;
							}
						}
					}else{
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample32F *ptr = dithererSrc32 + channel;
							TSample32S *outptr = ((TSample32S*)dest) + channel;
							for( size_t frame=0; frame<nFrames; frame++ ){
								*outptr = m_Dither32F_32S[channel].Dither( *ptr * 2147483647.0f );
								ptr+=m_uChannels;
								outptr+=m_uChannels;
							}
						}
					}
					break;
				case kInt8S:
					if( use64 ){
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample64F *ptr = dithererSrc64 + channel;
							TSample8S *outptr = ((TSample8S*)dest) + channel;
							for( size_t frame=0; frame<nFrames; frame++ ){
								*outptr = m_Dither64F_8S[channel].Dither( *ptr * 128.0 );
								ptr+=m_uChannels;
								outptr+=m_uChannels;
							}
						}
					}else{
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample32F *ptr = dithererSrc32 + channel;
							TSample8S *outptr = ((TSample8S*)dest) + channel;
							for( size_t frame=0; frame<nFrames; frame++ ){
								*outptr = m_Dither32F_8S[channel].Dither( *ptr * 128.0f );
								ptr+=m_uChannels;
								outptr+=m_uChannels;
							}
						}
					}
					break;
				case kInt8U:
					if( use64 ){
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample64F *ptr = dithererSrc64 + channel;
							TSample8U *outptr = ((TSample8U*)dest) + channel;
							for( size_t frame=0; frame<nFrames; frame++ ){
								*outptr = ConvertSample8Sto8U( m_Dither64F_8S[channel].Dither( *ptr * 128.0 ) );
								ptr+=m_uChannels;
								outptr+=m_uChannels;
							}
						}
					}else{
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample32F *ptr = dithererSrc32 + channel;
							TSample8U *outptr = ((TSample8U*)dest) + channel;
							for( size_t frame=0; frame<nFrames; frame++ ){
								*outptr = ConvertSample8Sto8U( m_Dither32F_8S[channel].Dither( *ptr * 128.0f ) );
								ptr+=m_uChannels;
								outptr+=m_uChannels;
							}
						}
					}
					break;
				case kInt20S:
					if( use64 ){
						unsigned frameSize = m_uChannels*3;
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample64F *ptr = dithererSrc64 + channel;
							unsigned char *outptr = ((unsigned char *)dest) + 3*channel;							
							for( size_t frame=0; frame<nFrames; frame++ ){
								ConvertSample32to20NoScaling( m_Dither64F_20S[channel].Dither( *ptr * 524287.0 ), outptr );
								ptr+=m_uChannels;
								outptr+=frameSize;
							}
						}
					}else{
						unsigned frameSize = m_uChannels*3;
						for( unsigned channel=0; channel<m_uChannels; channel++ ){
							const TSample32F *ptr = dithererSrc32 + channel;
							unsigned char *outptr = ((unsigned char *)dest) + 3*channel;							
							for( size_t frame=0; frame<nFrames; frame++ ){
								ConvertSample32to20NoScaling( m_Dither32F_20S[channel].Dither( *ptr * 524287.0f ), outptr );
								ptr+=m_uChannels;
								outptr+=frameSize;
							}
						}
					}
					break;
				default:
					VTDEBUGMSGT("Invalid destination type in dithering mode.");
					throw _T("Invalid destination type in dithering mode.");
			}
		}
	}
}

bool CAudioBitConverter::UsesDirectConversion() const
{
	if( m_SrcType==m_DestType ) // Copying job
		return true;
	if( m_DestType==kFP32 || m_DestType==kFP64 ) // Destination is floating point
		return true;
	if( m_DitheringMethod!=kDitheringNone ) // Dithering is desired, destination is int => must dither, cannot do direct conversion
		return false;
	else{
		// Dithering is OFF. We can also do direct conversion from FP to int.
		if(	m_SrcType==kFP32 || m_SrcType==kFP64 )
			return true;
	}
	return false;
}




class CVerifier{
public:
	CVerifier(){
		VTASSERT2T(
			sizeof( TSample8S )  == 1 &&
			sizeof( TSample16S ) == 2 &&
			sizeof( TSample32S ) == 4 &&
			sizeof( TSample32F ) == 4 &&
			sizeof( TSample64F ) == 8, 
			"Sample sizes are incorrect, please check the project build system!" );
	}
};

static CVerifier TheVerifier;



