//-----------------------------------------------------------------------------
//
// BASE64 encoding/decoding wrapper (uses b64 library)
//
//-----------------------------------------------------------------------------

/*
	VT Framework License (BSL 1.0)

	Copyright (C) 2006 Sergey A. Galin 
	E-mail: sergey.galin@gmail.com

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/

#ifndef VT_BASE64_HPP
#define VT_BASE64_HPP

#include "vt_portable.hpp"

// -------------------------------------------------------------------------
// Configuratiuon
// -------------------------------------------------------------------------

// Select which  library to use (we support only B64 so far)
#define USE_B64

// Define LSString functions?
#if !defined(USE_LSSTRING) && defined(USE_LSGUI)
	#define USE_LSSTRING
	#include "LSString.h"
#endif


#if defined(USE_B64)
	#include "../../Lib/include/b64.h"
#endif


// -------------------------------------------------------------------------
// Prototypes
// -------------------------------------------------------------------------

namespace VT {

	// The functions allocate memory using calloc() and return size of 
	// the binary output in out_size.
	// Call free() to release the memory.
	inline char *Encode(const void *in, size_t in_size);
	inline void *Decode(const char *in, size_t *out_size);

	#if defined(USE_LSSTRING)
		inline LString LEncode(const void *in, size_t in_size);
		inline void* LDecode(const LString &str, size_t *out_size);
	#endif







	// -------------------------------------------------------------------------
	// Implementation
	// -------------------------------------------------------------------------

	#if defined(USE_B64)
	char *VT::Encode(const void *in, size_t in_size)
	{
		// Ask for length required
		size_t cch=b64::b64_encode2(in, in_size, NULL, 0, b64::B64_F_LINE_LEN_64, 64, NULL); 
		char *enc=(char*)calloc(cch+1, sizeof(char));
		if(!enc)
			return NULL;
		b64::b64_encode2(in, in_size, enc, cch, b64::B64_F_LINE_LEN_64, 64, NULL);	
		enc[cch]=0;
		return enc;
	}

	void *VT::Decode(const char *in, size_t* out_size)
	{
		size_t cch = strlen(in);
		// Ask for size required
		*out_size = b64::b64_decode(in, cch, NULL, 0);
		char *dec = (char*)calloc(*out_size, 1);
		if(!dec)
			return NULL;
		b64::b64_decode(in, cch, dec, *out_size);
		return dec;
	}

	#endif

	#if defined(USE_LSSTRING)
	LString VT::LEncode(const void *in, size_t in_size)
	{
		char *out = VT::Encode(in, in_size);
		if(!out)
			return LString(_T(""));
		LString ret(out);
		free(out);
		return ret;
	}

	void *VT::LDecode(const LString &str, size_t *out_size)
	{
		return VT::Decode((LPCSTR)str.c_str(), out_size);
	}
	#endif

} // namespace

#endif

