//-----------------------------------------------------------------------------
//
// MP3 decoder wrapper, currently uses libmpg123.
//
//-----------------------------------------------------------------------------

/*
	VT Framework License (BSL 1.0)

	Copyright (C) 2007 Sergey A. Galin 
	E-mail: sergey.galin@gmail.com

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/


// Don't forget to link to libmpg123 library, e.g.:
// #pragma comment(lib, "../ThirdParty/libmpg123/libmpg123-0.lib")


#if !defined(VT_MP3DECODE_H)
#define VT_MP3DECODE_H

#include <stdio.h>
#include "vt_portable.hpp"

namespace VT {
	namespace MP3FileDecoder {

		// Typedef for a function which should display decoding progress to s
		typedef void TCallback(long samples, long currentFrame, long framesLeft, 
			double posSeconds, double leftSeconds);
		typedef TCallback* PTCallback;

		// Number of blocks to process between callbacks
		static const long kCallbackPeriod = 32;

		enum TDecoderResult
		{
			kOK,
			kUnknownError,
			kAborted,
			kEngineError,
			kInfileError,
			kOutfileError,
			kPrematureEOF,
			kUnknownFormat,
			kSystemError
		};

		// Decode MP3 infile to signed 16-bit PCM outfile.
		// The callback function can be used to display some progress indicators to user.
		// Use AbortDecode() to terminate the process.
		// If returned result is not kOK, use GetLastErrorMessage() to get detailed
		// textual description of the problem.

		TDecoderResult Decode(char *infile, char *outfile, PTCallback callback = NULL);
		TDecoderResult GetLength(char *file, double *sec, int *chan);

		#if defined(_UNICODE) && defined(_WIN32)
			// This function not only allows to pass VT_UNICODE file names to the DLL,
			// but also handles the VT_UNICODE-ANSI conversion problem, by creating and
			// passing short file names to Decode(char*, char*...).
			TDecoderResult Decode(LPCTSTR infile, LPCTSTR outfile, PTCallback callback = NULL);
			TDecoderResult GetLength(LPCTSTR file, double *sec, int *chan);
		#endif

		LPCVTCHAR GetLastErrorMessage();
		LPCVTCHAR GetErrorMessageHeader(TDecoderResult code);
		void AbortDecoding();
		bool DecodingAborted();

	} // namespace MP3FileDecoder
} // namespace VT

#endif

