//-----------------------------------------------------------------------------
//
// C++ Cross-Compilation Portability layer
//
//-----------------------------------------------------------------------------

/*
	VT Framework License (BSL 1.0)

	Copyright (C) 2006 Sergey A. Galin 
	E-mail: sergey.galin@gmail.com

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/


#include <ctype.h>
#include "vt_portable.hpp"

int vt_argc=0;
const char* const* vt_argv=NULL;

void VTCheckGlobalArgs()
{
	// !!!! To do: update this for some smarter behaviour under Windows
    if( !vt_argc || !vt_argv ){
		perror("VT: set global args via VTSetGlobalArgs()!\n");
		exit(255);    
    }
}


int VTASCII2VTCHAR(const char* src, LPVTCHAR dest, int maxLen, VTCHAR replaceChar, int *nReplacedChars)
{
	int i;
	if( nReplacedChars!=NULL )
		(*nReplacedChars) = 0;	
	for(i=0; (*src) && i<maxLen; i++, src++)
		if( ((*src) & char(0x7F)) != (*src) ){
			if( replaceChar )
				*(dest++) = replaceChar;
			if( nReplacedChars!=NULL )
				(*nReplacedChars)++;
		}else
			*(dest++) = VTCHAR(*src) & VTCHAR(0x7F);
	*dest = 0;
	return i;
}

int VTVTCHAR2ASCII(LPCVTCHAR src, char *dest, int maxLen, char replaceChar, int *nReplacedChars)
{
	int i;
	if( nReplacedChars!=NULL )
		(*nReplacedChars) = 0;	
	for(i=0; (*src) && i<maxLen; i++, src++)
		if( ((*src) & VTCHAR(0x7F)) != (*src) ){
			if( replaceChar )
				*(dest++) = replaceChar;
			if( nReplacedChars!=NULL )
				(*nReplacedChars)++;
		}else
			*(dest++) = char(*src) & char(0x7F);
	*dest = 0;
	return i;
}


int VTStrICmp(LPCVTCHAR s1, LPCVTCHAR s2)
{
	for(; *s1 && *s2; s1++, s2++){
		register int diff = VTTOUPPER(*s1)-VTTOUPPER(*s2);
		if( diff )
			return diff; 	
	}
	// Here, at least *s1 or *s2 is zero, so we don't
	// have to care about character case already.
	register int diff=int(*s2)-int(*s1);
	return diff;
}

int VTStrNICmp(LPCVTCHAR s1, LPCVTCHAR s2, size_t n)
{
	if( !n ) 
		return 0;
	for(; *s1 && *s2; s1++, s2++){
		register int diff = VTTOUPPER(*s1)-VTTOUPPER(*s2);
		if( diff )
			return diff; 
		n--;
		if( !n )
			return 0;
	}
	// Here, at least *s1 or *s2 is zero, so we don't
	// have to care about character case already.
	register int diff=int(*s2)-int(*s1);
	return diff;
}

FOURCC VTMakeFOURCC(unsigned char a, unsigned char b, unsigned char c, unsigned char d)
{
	return DWORD(a) | (DWORD(b)<<8) | (DWORD(c)<<16) | (DWORD(d)<<24); 
}

FOURCC VTMakeFOURCC(const char* str)
{
	unsigned char buf[4]; 
	(*((DWORD*)buf))=DWORD(0x20202020);
	for(int i=0; str[i] && i<4; i++) 
		buf[i]=(unsigned char)str[i];
	return VTMakeFOURCC( buf[0], buf[1], buf[2], buf[3] ); 
}

void VTFOURCC2Str(FOURCC code, char* str)
{
	*((FOURCC*)str) = code;
	str[4] = 0;
}

const char* VTFOURCC2Str(FOURCC code)
{
	static char str[5];
	VTFOURCC2Str( code, str );
	return str;
}



class CVTCompliationVerifier
{
public:
    CVTCompliationVerifier();
} gVerifier;

CVTCompliationVerifier::CVTCompliationVerifier()
{
    if( 
			sizeof(BYTE)!=1 || 
			sizeof(WORD)!=2 || 
			sizeof(DWORD)!=4 || 
			sizeof(SDWORD)!=4 || 
			sizeof(SIGNED_WORD)!=2	)
	{
		perror("Bad compiler options! Check Makefile and/or portable library headers.");
		exit(255);
    }

	if( sizeof(DWORD64)!=8 ){	
		perror("Bad DWORD64 size! Check Makefile and/or portable library headers.");
		exit(255);
	}
}




