//-----------------------------------------------------------------------------
//
// Some helpers to ease handling of resource loading.
//
//-----------------------------------------------------------------------------

/*
	VT Framework License (BSL 1.0)

	Copyright (C) 2007 Sergey A. Galin 
	E-mail: sergey.galin@gmail.com

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/

#if !defined(VT_RESOURCELOADERS_H)
#define VT_RESOURCELOADERS_H

#include "vt_portable.hpp"

#if defined(USE_LSGUI)
	#include "LSWindows.h"
#endif

// Icon loader, intended useage:
// 1. Create global or static CIconLoader object for an icon.
// 2. init() it after hInstance is available, e.g. in class constructor 
//    of a non-static object, in WinMain, and etc. If the icon is already
//    loaded, calling init() won't do any effect.
// 3. Use icon handle retrieved by getIcon() to draw the icon.

namespace VT {

	class CIconLoader 
	{
	public:
		#if defined(VT_WINDOWS)
			CIconLoader(HINSTANCE hInstance=NULL, int resourceId=0):
				m_hIcon(NULL), m_hInstance(hInstance), m_resourceId(resourceId)
			{}
		#endif

		~CIconLoader();

		#if defined(VT_WINDOWS)
			VTINLINE void init(HINSTANCE hInstance, int resourceId){
				m_hInstance = hInstance;
				m_resourceId = resourceId;
			}
		
			#if defined(USE_LSGUI)
				VTINLINE void init(int resourceId){ init(lsData.hInstance, resourceId); }
			#endif
		
			HICON getIcon();
		#endif

	private:
		#if defined(VT_WINDOWS)
			HINSTANCE m_hInstance;
			int m_resourceId;
			HICON m_hIcon;
		#endif
	};

} // namespace







#endif


