//-----------------------------------------------------------------------------
//
// Zlib wrapper.
//
//-----------------------------------------------------------------------------

/*
	VT Framework License (BSL 1.0)

	Copyright (C) 2006 Sergey A. Galin 
	E-mail: sergey.galin@gmail.com

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/

#ifndef VT_ZLIB_H
#define VT_ZLIB_H

#include "vt_portable.hpp"

namespace VT {
	namespace ZLib {
		enum TCompression{ kBest, kFast, kDefault };

		// In-memory, single-block compression & decompression.
		// The functions allocate memory using calloc() and return size of 
		// the binary		 output in out_size. Use free() to release the memory.
		//
		// NOTE 1: the compressed data contains additional fields
		// including signature and uncompressed data length to speed up 
		// decompressing, so the data cannot be passed directly to zlib routines.
		//
		// NOTE 2: if VT_EXCEPTIONS is defined, both functions will throw
		// exceptions of type TCHAR* in case of errors.
		//
		void *Compress(const void *in, size_t in_size, size_t *out_size, TCompression level=ZLib::kDefault);
		
		// Note: If you pass a block which is too small or contains no signature to it,
		// input data will be copied to output.
		void *Decompress(const void *in, size_t in_size, size_t *out_size);

		// Use this function to ensure that the data looks like compressed data
		// (i.e. contains proper signature & has minimum size).
		bool IsCompressedData(const void *in, size_t in_size);

	} // namespace
} // namespace


#endif



